# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sarus_data_spec/protobuf/dataset.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='sarus_data_spec/protobuf/dataset.proto',
  package='sarus_data_spec',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n&sarus_data_spec/protobuf/dataset.proto\x12\x0fsarus_data_spec\"\xa3\x07\n\x07\x44\x61taset\x12\x0c\n\x04uuid\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0b\n\x03\x64oc\x18\x03 \x01(\t\x12+\n\x04spec\x18\x04 \x01(\x0b\x32\x1d.sarus_data_spec.Dataset.Spec\x12<\n\nproperties\x18\x05 \x03(\x0b\x32(.sarus_data_spec.Dataset.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x8d\x02\n\x04Spec\x12;\n\x0btransformed\x18\x01 \x01(\x0b\x32$.sarus_data_spec.Dataset.TransformedH\x00\x12-\n\x04\x66ile\x18\x02 \x01(\x0b\x32\x1d.sarus_data_spec.Dataset.FileH\x00\x12/\n\x05\x66iles\x18\x03 \x01(\x0b\x32\x1e.sarus_data_spec.Dataset.FilesH\x00\x12\x33\n\x07\x61rchive\x18\x04 \x01(\x0b\x32 .sarus_data_spec.Dataset.ArchiveH\x00\x12+\n\x03sql\x18\x05 \x01(\x0b\x32\x1c.sarus_data_spec.Dataset.SqlH\x00\x42\x06\n\x04spec\x1a\xbd\x01\n\x0bTransformed\x12\x11\n\ttransform\x18\x01 \x01(\t\x12\x11\n\targuments\x18\x02 \x03(\t\x12Q\n\x0fnamed_arguments\x18\x03 \x03(\x0b\x32\x38.sarus_data_spec.Dataset.Transformed.NamedArgumentsEntry\x1a\x35\n\x13NamedArgumentsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a#\n\x04\x46ile\x12\x0e\n\x06\x66ormat\x18\x01 \x01(\t\x12\x0b\n\x03uri\x18\x02 \x01(\t\x1a,\n\x05\x46iles\x12\x0e\n\x06\x66ormat\x18\x01 \x01(\t\x12\x13\n\x0buri_pattern\x18\x02 \x01(\t\x1a>\n\x07\x41rchive\x12\x0e\n\x06\x66ormat\x18\x01 \x01(\t\x12\x16\n\x0e\x61rchive_format\x18\x02 \x01(\t\x12\x0b\n\x03uri\x18\x03 \x01(\t\x1an\n\x03Sql\x12\x0b\n\x03uri\x18\x01 \x01(\t\x12\x32\n\x06tables\x18\x02 \x03(\x0b\x32\".sarus_data_spec.Dataset.Sql.Table\x1a&\n\x05Table\x12\x0e\n\x06schema\x18\x01 \x01(\t\x12\r\n\x05table\x18\x02 \x01(\tb\x06proto3'
)




_DATASET_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='sarus_data_spec.Dataset.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='sarus_data_spec.Dataset.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Dataset.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=219,
  serialized_end=268,
)

_DATASET_SPEC = _descriptor.Descriptor(
  name='Spec',
  full_name='sarus_data_spec.Dataset.Spec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='transformed', full_name='sarus_data_spec.Dataset.Spec.transformed', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='file', full_name='sarus_data_spec.Dataset.Spec.file', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='files', full_name='sarus_data_spec.Dataset.Spec.files', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='archive', full_name='sarus_data_spec.Dataset.Spec.archive', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sql', full_name='sarus_data_spec.Dataset.Spec.sql', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='spec', full_name='sarus_data_spec.Dataset.Spec.spec',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=271,
  serialized_end=540,
)

_DATASET_TRANSFORMED_NAMEDARGUMENTSENTRY = _descriptor.Descriptor(
  name='NamedArgumentsEntry',
  full_name='sarus_data_spec.Dataset.Transformed.NamedArgumentsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='sarus_data_spec.Dataset.Transformed.NamedArgumentsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Dataset.Transformed.NamedArgumentsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=679,
  serialized_end=732,
)

_DATASET_TRANSFORMED = _descriptor.Descriptor(
  name='Transformed',
  full_name='sarus_data_spec.Dataset.Transformed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='transform', full_name='sarus_data_spec.Dataset.Transformed.transform', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='arguments', full_name='sarus_data_spec.Dataset.Transformed.arguments', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='named_arguments', full_name='sarus_data_spec.Dataset.Transformed.named_arguments', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DATASET_TRANSFORMED_NAMEDARGUMENTSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=543,
  serialized_end=732,
)

_DATASET_FILE = _descriptor.Descriptor(
  name='File',
  full_name='sarus_data_spec.Dataset.File',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='format', full_name='sarus_data_spec.Dataset.File.format', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uri', full_name='sarus_data_spec.Dataset.File.uri', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=734,
  serialized_end=769,
)

_DATASET_FILES = _descriptor.Descriptor(
  name='Files',
  full_name='sarus_data_spec.Dataset.Files',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='format', full_name='sarus_data_spec.Dataset.Files.format', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uri_pattern', full_name='sarus_data_spec.Dataset.Files.uri_pattern', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=771,
  serialized_end=815,
)

_DATASET_ARCHIVE = _descriptor.Descriptor(
  name='Archive',
  full_name='sarus_data_spec.Dataset.Archive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='format', full_name='sarus_data_spec.Dataset.Archive.format', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='archive_format', full_name='sarus_data_spec.Dataset.Archive.archive_format', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uri', full_name='sarus_data_spec.Dataset.Archive.uri', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=817,
  serialized_end=879,
)

_DATASET_SQL_TABLE = _descriptor.Descriptor(
  name='Table',
  full_name='sarus_data_spec.Dataset.Sql.Table',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='schema', full_name='sarus_data_spec.Dataset.Sql.Table.schema', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table', full_name='sarus_data_spec.Dataset.Sql.Table.table', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=953,
  serialized_end=991,
)

_DATASET_SQL = _descriptor.Descriptor(
  name='Sql',
  full_name='sarus_data_spec.Dataset.Sql',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uri', full_name='sarus_data_spec.Dataset.Sql.uri', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tables', full_name='sarus_data_spec.Dataset.Sql.tables', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DATASET_SQL_TABLE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=881,
  serialized_end=991,
)

_DATASET = _descriptor.Descriptor(
  name='Dataset',
  full_name='sarus_data_spec.Dataset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid', full_name='sarus_data_spec.Dataset.uuid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='sarus_data_spec.Dataset.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='doc', full_name='sarus_data_spec.Dataset.doc', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='spec', full_name='sarus_data_spec.Dataset.spec', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='sarus_data_spec.Dataset.properties', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DATASET_PROPERTIESENTRY, _DATASET_SPEC, _DATASET_TRANSFORMED, _DATASET_FILE, _DATASET_FILES, _DATASET_ARCHIVE, _DATASET_SQL, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=991,
)

_DATASET_PROPERTIESENTRY.containing_type = _DATASET
_DATASET_SPEC.fields_by_name['transformed'].message_type = _DATASET_TRANSFORMED
_DATASET_SPEC.fields_by_name['file'].message_type = _DATASET_FILE
_DATASET_SPEC.fields_by_name['files'].message_type = _DATASET_FILES
_DATASET_SPEC.fields_by_name['archive'].message_type = _DATASET_ARCHIVE
_DATASET_SPEC.fields_by_name['sql'].message_type = _DATASET_SQL
_DATASET_SPEC.containing_type = _DATASET
_DATASET_SPEC.oneofs_by_name['spec'].fields.append(
  _DATASET_SPEC.fields_by_name['transformed'])
_DATASET_SPEC.fields_by_name['transformed'].containing_oneof = _DATASET_SPEC.oneofs_by_name['spec']
_DATASET_SPEC.oneofs_by_name['spec'].fields.append(
  _DATASET_SPEC.fields_by_name['file'])
_DATASET_SPEC.fields_by_name['file'].containing_oneof = _DATASET_SPEC.oneofs_by_name['spec']
_DATASET_SPEC.oneofs_by_name['spec'].fields.append(
  _DATASET_SPEC.fields_by_name['files'])
_DATASET_SPEC.fields_by_name['files'].containing_oneof = _DATASET_SPEC.oneofs_by_name['spec']
_DATASET_SPEC.oneofs_by_name['spec'].fields.append(
  _DATASET_SPEC.fields_by_name['archive'])
_DATASET_SPEC.fields_by_name['archive'].containing_oneof = _DATASET_SPEC.oneofs_by_name['spec']
_DATASET_SPEC.oneofs_by_name['spec'].fields.append(
  _DATASET_SPEC.fields_by_name['sql'])
_DATASET_SPEC.fields_by_name['sql'].containing_oneof = _DATASET_SPEC.oneofs_by_name['spec']
_DATASET_TRANSFORMED_NAMEDARGUMENTSENTRY.containing_type = _DATASET_TRANSFORMED
_DATASET_TRANSFORMED.fields_by_name['named_arguments'].message_type = _DATASET_TRANSFORMED_NAMEDARGUMENTSENTRY
_DATASET_TRANSFORMED.containing_type = _DATASET
_DATASET_FILE.containing_type = _DATASET
_DATASET_FILES.containing_type = _DATASET
_DATASET_ARCHIVE.containing_type = _DATASET
_DATASET_SQL_TABLE.containing_type = _DATASET_SQL
_DATASET_SQL.fields_by_name['tables'].message_type = _DATASET_SQL_TABLE
_DATASET_SQL.containing_type = _DATASET
_DATASET.fields_by_name['spec'].message_type = _DATASET_SPEC
_DATASET.fields_by_name['properties'].message_type = _DATASET_PROPERTIESENTRY
DESCRIPTOR.message_types_by_name['Dataset'] = _DATASET
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Dataset = _reflection.GeneratedProtocolMessageType('Dataset', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASET_PROPERTIESENTRY,
    '__module__' : 'sarus_data_spec.protobuf.dataset_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Dataset.PropertiesEntry)
    })
  ,

  'Spec' : _reflection.GeneratedProtocolMessageType('Spec', (_message.Message,), {
    'DESCRIPTOR' : _DATASET_SPEC,
    '__module__' : 'sarus_data_spec.protobuf.dataset_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Dataset.Spec)
    })
  ,

  'Transformed' : _reflection.GeneratedProtocolMessageType('Transformed', (_message.Message,), {

    'NamedArgumentsEntry' : _reflection.GeneratedProtocolMessageType('NamedArgumentsEntry', (_message.Message,), {
      'DESCRIPTOR' : _DATASET_TRANSFORMED_NAMEDARGUMENTSENTRY,
      '__module__' : 'sarus_data_spec.protobuf.dataset_pb2'
      # @@protoc_insertion_point(class_scope:sarus_data_spec.Dataset.Transformed.NamedArgumentsEntry)
      })
    ,
    'DESCRIPTOR' : _DATASET_TRANSFORMED,
    '__module__' : 'sarus_data_spec.protobuf.dataset_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Dataset.Transformed)
    })
  ,

  'File' : _reflection.GeneratedProtocolMessageType('File', (_message.Message,), {
    'DESCRIPTOR' : _DATASET_FILE,
    '__module__' : 'sarus_data_spec.protobuf.dataset_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Dataset.File)
    })
  ,

  'Files' : _reflection.GeneratedProtocolMessageType('Files', (_message.Message,), {
    'DESCRIPTOR' : _DATASET_FILES,
    '__module__' : 'sarus_data_spec.protobuf.dataset_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Dataset.Files)
    })
  ,

  'Archive' : _reflection.GeneratedProtocolMessageType('Archive', (_message.Message,), {
    'DESCRIPTOR' : _DATASET_ARCHIVE,
    '__module__' : 'sarus_data_spec.protobuf.dataset_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Dataset.Archive)
    })
  ,

  'Sql' : _reflection.GeneratedProtocolMessageType('Sql', (_message.Message,), {

    'Table' : _reflection.GeneratedProtocolMessageType('Table', (_message.Message,), {
      'DESCRIPTOR' : _DATASET_SQL_TABLE,
      '__module__' : 'sarus_data_spec.protobuf.dataset_pb2'
      # @@protoc_insertion_point(class_scope:sarus_data_spec.Dataset.Sql.Table)
      })
    ,
    'DESCRIPTOR' : _DATASET_SQL,
    '__module__' : 'sarus_data_spec.protobuf.dataset_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Dataset.Sql)
    })
  ,
  'DESCRIPTOR' : _DATASET,
  '__module__' : 'sarus_data_spec.protobuf.dataset_pb2'
  # @@protoc_insertion_point(class_scope:sarus_data_spec.Dataset)
  })
_sym_db.RegisterMessage(Dataset)
_sym_db.RegisterMessage(Dataset.PropertiesEntry)
_sym_db.RegisterMessage(Dataset.Spec)
_sym_db.RegisterMessage(Dataset.Transformed)
_sym_db.RegisterMessage(Dataset.Transformed.NamedArgumentsEntry)
_sym_db.RegisterMessage(Dataset.File)
_sym_db.RegisterMessage(Dataset.Files)
_sym_db.RegisterMessage(Dataset.Archive)
_sym_db.RegisterMessage(Dataset.Sql)
_sym_db.RegisterMessage(Dataset.Sql.Table)


_DATASET_PROPERTIESENTRY._options = None
_DATASET_TRANSFORMED_NAMEDARGUMENTSENTRY._options = None
# @@protoc_insertion_point(module_scope)
