# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sarus_data_spec/protobuf/proto_container.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='sarus_data_spec/protobuf/proto_container.proto',
  package='sarus_data_spec',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n.sarus_data_spec/protobuf/proto_container.proto\x12\x0fsarus_data_spec\x1a\x19google/protobuf/any.proto\"\xae\x01\n\x0eProtoContainer\x12$\n\x06protos\x18\x01 \x03(\x0b\x32\x14.google.protobuf.Any\x12\x43\n\nproperties\x18\x02 \x03(\x0b\x32/.sarus_data_spec.ProtoContainer.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x62\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])




_PROTOCONTAINER_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='sarus_data_spec.ProtoContainer.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='sarus_data_spec.ProtoContainer.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.ProtoContainer.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=220,
  serialized_end=269,
)

_PROTOCONTAINER = _descriptor.Descriptor(
  name='ProtoContainer',
  full_name='sarus_data_spec.ProtoContainer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='protos', full_name='sarus_data_spec.ProtoContainer.protos', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='sarus_data_spec.ProtoContainer.properties', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_PROTOCONTAINER_PROPERTIESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=95,
  serialized_end=269,
)

_PROTOCONTAINER_PROPERTIESENTRY.containing_type = _PROTOCONTAINER
_PROTOCONTAINER.fields_by_name['protos'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_PROTOCONTAINER.fields_by_name['properties'].message_type = _PROTOCONTAINER_PROPERTIESENTRY
DESCRIPTOR.message_types_by_name['ProtoContainer'] = _PROTOCONTAINER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProtoContainer = _reflection.GeneratedProtocolMessageType('ProtoContainer', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _PROTOCONTAINER_PROPERTIESENTRY,
    '__module__' : 'sarus_data_spec.protobuf.proto_container_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.ProtoContainer.PropertiesEntry)
    })
  ,
  'DESCRIPTOR' : _PROTOCONTAINER,
  '__module__' : 'sarus_data_spec.protobuf.proto_container_pb2'
  # @@protoc_insertion_point(class_scope:sarus_data_spec.ProtoContainer)
  })
_sym_db.RegisterMessage(ProtoContainer)
_sym_db.RegisterMessage(ProtoContainer.PropertiesEntry)


_PROTOCONTAINER_PROPERTIESENTRY._options = None
# @@protoc_insertion_point(module_scope)
